Stan Math Library: stan/math/prim/mat/prob/multi_normal_cholesky_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

multi_normal_cholesky_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_MAT_PROB_MULTI_NORMAL_CHOLESKY_LOG_HPP
2 #define STAN_MATH_PRIM_MAT_PROB_MULTI_NORMAL_CHOLESKY_LOG_HPP
3
4 #include <stan/math/prim/mat/fun/columns_dot_product.hpp>
5 #include <stan/math/prim/mat/fun/columns_dot_self.hpp>
6 #include <stan/math/prim/mat/fun/dot_product.hpp>
7 #include <stan/math/prim/mat/fun/dot_self.hpp>
8 #include <stan/math/prim/mat/fun/log.hpp>
9 #include <stan/math/prim/mat/fun/log_determinant.hpp>
10 #include <stan/math/prim/mat/fun/mdivide_left_spd.hpp>
11 #include <stan/math/prim/mat/fun/mdivide_left_tri_low.hpp>
12 #include <stan/math/prim/mat/fun/multiply.hpp>
13 #include <stan/math/prim/mat/fun/subtract.hpp>
14 #include <stan/math/prim/mat/fun/sum.hpp>
15 #include <stan/math/prim/mat/meta/VectorViewMvt.hpp>
16 #include <stan/math/prim/scal/err/check_size_match.hpp>
17 #include <stan/math/prim/scal/err/check_finite.hpp>
18 #include <stan/math/prim/scal/err/check_not_nan.hpp>
19 #include <stan/math/prim/scal/fun/constants.hpp>
20 #include <stan/math/prim/scal/meta/max_size_mvt.hpp>
21 #include <stan/math/prim/scal/meta/include_summand.hpp>
22 #include <boost/random/normal_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24
25 namespace stan {
26 namespace math {
44 template <bool propto,
45 typename T_y, typename T_loc, typename T_covar>
46 typename return_type<T_y, T_loc, T_covar>::type
47 multi_normal_cholesky_log(const T_y& y,
48 const T_loc& mu,
49 const T_covar& L) {
50 static const char* function("stan::math::multi_normal_cholesky_log");
51 typedef typename scalar_type<T_covar>::type T_covar_elem;
52 typedef typename return_type<T_y, T_loc, T_covar>::type lp_type;
53 lp_type lp(0.0);
54
55 using stan::math::mdivide_left_tri_low;
56 using stan::math::dot_self;
57 using stan::math::multiply;
58 using stan::math::subtract;
59 using stan::math::sum;
60
61 using stan::math::check_size_match;
62 using stan::math::check_finite;
63 using stan::math::check_not_nan;
64
65 VectorViewMvt<const T_y> y_vec(y);
66 VectorViewMvt<const T_loc> mu_vec(mu);
67 // size of std::vector of Eigen vectors
68 size_t size_vec = max_size_mvt(y, mu);
69
70 // Check if every vector of the array has the same size
71 int size_y = y_vec[0].size();
72 int size_mu = mu_vec[0].size();
73 if (size_vec > 1) {
74 int size_y_old = size_y;
75 int size_y_new;
76 for (size_t i = 1, size_ = length_mvt(y); i < size_; i++) {
77 int size_y_new = y_vec[i].size();
78 check_size_match(function,
79 "Size of one of the vectors of "
80 "the random variable", size_y_new,
81 "Size of another vector of the "
82 "random variable", size_y_old);
83 size_y_old = size_y_new;
84 }
85 int size_mu_old = size_mu;
86 int size_mu_new;
87 for (size_t i = 1, size_ = length_mvt(mu); i < size_; i++) {
88 int size_mu_new = mu_vec[i].size();
89 check_size_match(function,
90 "Size of one of the vectors of "
91 "the location variable", size_mu_new,
92 "Size of another vector of the "
93 "location variable", size_mu_old);
94 size_mu_old = size_mu_new;
95 }
96 (void) size_y_old;
97 (void) size_y_new;
98 (void) size_mu_old;
99 (void) size_mu_new;
100 }
101
102 check_size_match(function,
103 "Size of random variable", size_y,
104 "size of location parameter", size_mu);
105 check_size_match(function,
106 "Size of random variable", size_y,
107 "rows of covariance parameter", L.rows());
108 check_size_match(function,
109 "Size of random variable", size_y,
110 "columns of covariance parameter", L.cols());
111
112 for (size_t i = 0; i < size_vec; i++) {
113 check_finite(function, "Location parameter", mu_vec[i]);
114 check_not_nan(function, "Random variable", y_vec[i]);
115 }
116
117 if (size_y == 0)
118 return lp;
119
120 if (include_summand<propto>::value)
121 lp += NEG_LOG_SQRT_TWO_PI * size_y * size_vec;
122
123 if (include_summand<propto, T_covar_elem>::value)
124 lp -= L.diagonal().array().log().sum() * size_vec;
125
126 if (include_summand<propto, T_y, T_loc, T_covar_elem>::value) {
127 lp_type sum_lp_vec(0.0);
128 for (size_t i = 0; i < size_vec; i++) {
129 Eigen::Matrix<typename return_type<T_y, T_loc>::type,
130 Eigen::Dynamic, 1> y_minus_mu(size_y);
131 for (int j = 0; j < size_y; j++)
132 y_minus_mu(j) = y_vec[i](j)-mu_vec[i](j);
133 Eigen::Matrix<typename return_type<T_y, T_loc, T_covar>::type,
134 Eigen::Dynamic, 1>
135 half(mdivide_left_tri_low(L, y_minus_mu));
136 // FIXME: this code does not compile. revert after fixing subtract()
137 // Eigen::Matrix<typename
138 // boost::math::tools::promote_args<T_covar,
139 // typename value_type<T_loc>::type,
140 // typename value_type<T_y>::type>::type>::type,
141 // Eigen::Dynamic, 1>
142 // half(mdivide_left_tri_low(L, subtract(y, mu)));
143 sum_lp_vec += dot_self(half);
144 }
145 lp -= 0.5*sum_lp_vec;
146 }
147 return lp;
148 }
149
150 template <typename T_y, typename T_loc, typename T_covar>
151 inline
152 typename return_type<T_y, T_loc, T_covar>::type
153 multi_normal_cholesky_log(const T_y& y, const T_loc& mu, const T_covar& L) {
154 return multi_normal_cholesky_log<false>(y, mu, L);
155 }
156
157 }
158 }
159 #endif
check_not_nan.hpp
stan::math::sum
fvar< T > sum(const std::vector< fvar< T > > &m)
Return the sum of the entries of the specified standard vector.
Definition: sum.hpp:20
multiply.hpp
max_size_mvt.hpp
stan::max_size_mvt
size_t max_size_mvt(const T1 &x1, const T2 &x2)
Definition: max_size_mvt.hpp:10
mdivide_left_spd.hpp
log_determinant.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_size_match.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::subtract
Eigen::Matrix< typename boost::math::tools::promote_args< T1, T2 >::type, R, C > subtract(const Eigen::Matrix< T1, R, C > &m1, const Eigen::Matrix< T2, R, C > &m2)
Return the result of subtracting the second specified matrix from the first specified matrix...
Definition: subtract.hpp:27
VectorViewMvt.hpp
stan::math::multiply
Eigen::Matrix< fvar< T >, R1, C1 > multiply(const Eigen::Matrix< fvar< T >, R1, C1 > &m, const fvar< T > &c)
Definition: multiply.hpp:21
stan::scalar_type::type
scalar_type_helper< is_vector< T >::value, T >::type type
Definition: scalar_type.hpp:35
stan::math::dot_self
fvar< T > dot_self(const Eigen::Matrix< fvar< T >, R, C > &v)
Definition: dot_self.hpp:16
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
log.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
columns_dot_product.hpp
stan::math::multi_normal_cholesky_log
return_type< T_y, T_loc, T_covar >::type multi_normal_cholesky_log(const T_y &y, const T_loc &mu, const T_covar &L)
The log of the multivariate normal density for the given y, mu, and a Cholesky factor L of the varian...
Definition: multi_normal_cholesky_log.hpp:47
size_
size_t size_
Definition: dot_self.hpp:18
stan::math::check_size_match
bool check_size_match(const char *function, const char *name_i, T_size1 i, const char *name_j, T_size2 j)
Return true if the provided sizes match.
Definition: check_size_match.hpp:30
dot_product.hpp
stan::math::NEG_LOG_SQRT_TWO_PI
const double NEG_LOG_SQRT_TWO_PI
Definition: constants.hpp:184
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
check_finite.hpp
stan::math::mdivide_left_tri_low
Eigen::Matrix< fvar< T >, R1, C1 > mdivide_left_tri_low(const Eigen::Matrix< fvar< T >, R1, C1 > &A, const Eigen::Matrix< fvar< T >, R2, C2 > &b)
Definition: mdivide_left_tri_low.hpp:22
sum.hpp
stan::VectorViewMvt
Definition: VectorViewMvt.hpp:16
dot_self.hpp
columns_dot_self.hpp
subtract.hpp
mdivide_left_tri_low.hpp
stan::length_mvt
size_t length_mvt(const Eigen::Matrix< T, R, C > &)
Definition: length_mvt.hpp:12

[ Stan Home Page ] © 2011–2016, Stan Development Team.